// RomanNumC.cpp : Defines the entry point for the console application.

//				Converts between Roman Numerals and Arabic Numbers
//
//		From Dr. Clifford A. Pickover's book, Wonders of Numbers, Oxford University Press, 2001.
//		Generates Arabic Numbers when user types in Roman Numerals
//		Generates Roman Numerals when user types in Arabic Numbers

#include "stdafx.h"
#include <iostream>		// for cout
#include <time.h>		// for _strdate and _strtime
#include <iomanip>		// for setiosflags and setprecision

#define nL	'\n'		// new line
#define dL	"\n\n"		// double new line

using namespace System;
using namespace System::Reflection;		// for Assembly. ...
using namespace std;

void AToR();
void TimeStamp();

// Global variables:
bool done;			// done flag for current search
char *cp;			// pointer to character in rw
char rn[1001];		// input string converted to upper case
char rw[1001];		// working form of Roman Numeral
int i;				// utility index
int len;			// length of input string
int maxL = 1001;	// max length of a Roman Numeral input or output
int toGo;			// number of unprocessed characters in rw
long an;			// the Arabic Number input or output

// ------- initialize
void init(void)
{	
	char name[]    = "RomanNumC - Converts between Roman Numerals and Arabic Numbers";
	char author1[] = "Copyright (c) 1981-2008 by author: Harry J. Smith,";
	char author2[] = "19628 Via Monte Dr., Saratoga, CA 95070.  All rights reserved.";
	String^ version;		// = "C++ Version x.x.xxxx.xxxxx"
	Version aVersion = Assembly::GetExecutingAssembly()->GetName()->Version->ToString();
	version = "C++ Console Version " + aVersion.ToString() + " ";
	cout << setiosflags(ios::fixed) << setprecision(4) << "";
	cout << nL << name << nL;	// show credits
	int lenV;
	lenV = version->LastIndexOf(" ");
	char sta[40];
	for (i = 0; i < lenV; i++)
		sta[i] = (char)version[i];
	sta[i] = 0;
	cout << sta << nL;
	cout << author1 << nL;
	cout << author2 << dL;
	cout << "Ran on: ";
	TimeStamp();
	cout << nL;
} // init

// The main entry point for the application RomanNumC
int main()
{
	init();
	while (true)
	{
		cout << nL << " Enter a Roman Numeral or an Arabic Number (Q to exit) ";
		cin >> rn;
		len = (int)strnlen(rn, maxL + 1);
		if (len > maxL)
		{
			len = maxL;
			if (rn[0] == 'Q') return 0;
			rn[0] = '0';
		}
		for (i = 0; i < len; i++)
		{
			rn[i] = toupper(rn[i]);
			if (rn[i] == 'Q') return 0;
		}
		if (rn[0] >= '0' && rn[0] <= '9')
		{
			if (len > 7)
			{
				cout << " <Number too large>" << nL;
			}
			else
			{
				an = 0;
				for (i = 0; i < len; i++)
				{
					an *= 10;
					an += (int)rn[i] - '0';
				}
				cout << " The Roman Numeral for " << an << " is ";
				AToR();
				cout << rw << nL;
			}
		}
		else
		{
			for (i = 0; i <= len; i++)
				rw[i] = rn[i];
			an = 0;
			cp = rw;
			toGo = len;
			for (i = 0, cp = rw; i < len && toGo > 1; i++, cp++)
			{
				if (*(cp) == 'C' && *(cp+1) == 'M')
				{
					*cp = ' ';
					i++;
					cp++;
					*cp = ' ';
					toGo -= 2;
					an = an + 900;
				}
			}
			for (i = 0, cp = rw; i < len && toGo > 1; i++, cp++)
			{
				if (*(cp) == 'C' && *(cp+1) == 'D')
				{
					*cp = ' ';
					i++;
					cp++;
					*cp = ' ';
					toGo -= 2;
					an = an + 400;
				}
			}
			for (i = 0, cp = rw; i < len && toGo > 1; i++, cp++)
			{
				if (*(cp) == 'X' && *(cp+1) == 'C')
				{
					*cp = ' ';
					i++;
					cp++;
					*cp = ' ';
					toGo -= 2;
					an = an + 90;
				}
			}
			for (i = 0, cp = rw; i < len && toGo > 1; i++, cp++)
			{
				if (*(cp) == 'X' && *(cp+1) == 'L')
				{
					*cp = ' ';
					i++;
					cp++;
					*cp = ' ';
					toGo -= 2;
					an = an + 40;
				}
			}
			for (i = 0, cp = rw; i < len && toGo > 1; i++, cp++)
			{
				if (*(cp) == 'I' && *(cp+1) == 'X')
				{
					*cp = ' ';
					i++;
					cp++;
					*cp = ' ';
					toGo -= 2;
					an = an + 9;
				}
			}
			for (i = 0, cp = rw; i < len && toGo > 1; i++, cp++)
			{
				if (*(cp) == 'I' && *(cp+1) == 'V')
				{
					*cp = ' ';
					i++;
					cp++;
					*cp = ' ';
					toGo -= 2;
					an = an + 4;
				}
			}
			done = false;
			while (!done && toGo > 0)
			{
				cp = strchr(rw, 'M');
				if (cp == NULL)			// NULL = (char*)0 = null pointer
					done = true;
				else
				{
					*cp = ' ';
					toGo--;
					an = an + 1000;
				}
			}
			done = false;
			while (!done && toGo > 0)
			{
				cp = strchr(rw, 'D');
				if (cp == NULL)
					done = true;
				else
				{
					*cp = ' ';
					toGo--;
					an = an + 500;
				}
			}
			done = false;
			while (!done && toGo > 0)
			{
				cp = strchr(rw, 'C');
				if (cp == NULL)
					done = true;
				else
				{
					*cp = ' ';
					toGo--;
					an = an + 100;
				}
			}
			done = false;
			while (!done && toGo > 0)
			{
				cp = strchr(rw, 'L');
				if (cp == NULL)
					done = true;
				else
				{
					*cp = ' ';
					toGo--;
					an = an + 50;
				}
			}
			done = false;
			while (!done && toGo > 0)
			{
				cp = strchr(rw, 'X');
				if (cp == NULL)
					done = true;
				else
				{
					*cp = ' ';
					toGo--;
					an = an + 10;
				}
			}
			done = false;
			while (!done && toGo > 0)
			{
				cp = strchr(rw, 'V');
				if (cp == NULL)
					done = true;
				else
				{
					*cp = ' ';
					toGo--;
					an = an + 5;
				}
			}
			done = false;
			while (!done && toGo > 0)
			{
				cp = strchr(rw, 'I');
				if (cp == NULL)
					done = true;
				else
				{
					*cp = ' ';
					toGo--;
					an = an + 1;
				}
			}
			cout << " The Arabic Number for " << rn << " is " << an << nL;
			AToR();
			if (strcmp(rw, rn) == 0)
			  cout << " The Roman Numeral is in the correct format" << nL;
			else
			  cout << " The correct format for this is " <<  rw << nL;
		}
	}
	TimeStamp();
	return 0;
} // main

void AToR()	// Convert from Arabic Number an to Roman Numeral rw
{
	String ^St = "";		// String used to build the Roman Numeral
	while (an >= 1000)
	{
		St = String::Concat(St, "M");
		an = an - 1000;
	}
	if (an >= 900) {St = String::Concat(St, "CM"); an = an - 900;}	// [0, 999]
	if (an >= 500) {St = String::Concat(St, "D"); an = an - 500;}	// [0, 899]
	if (an >= 400) {St = String::Concat(St, "CD"); an = an - 400;}	// [0, 499]
	while (an >= 100)												// [0, 399]
	{
		St = String::Concat(St, "C");
		an = an - 100;
	}
	if (an >= 90) {St = String::Concat(St, "XC"); an = an - 90;}	// [0, 99]
	if (an >= 50) {St = String::Concat(St, "L"); an = an - 50;}		// [0, 89]
	if (an >= 40) {St = String::Concat(St, "XL"); an = an - 40;}	// [0, 49]
	while (an >= 10)								// [0, 39]
	{
		St = String::Concat(St, "X");
		an = an - 10;
	}
	if (an >= 9) {St = String::Concat(St, "IX"); an = an - 9;}		// [0, 9]
	if (an >= 5) {St = String::Concat(St, "V"); an = an - 5;}		// [0, 8]
	if (an >= 4) {St = String::Concat(St, "IV"); an = an - 4;}		// [0, 4]
	while (an >= 1)													// [0, 3]
	{
		St = String::Concat(St, "I");
		an = an - 1;
	}
	rw[0] = 0;
	if (St->Length > maxL)
		cout << "<Number too large>";
	else
	{
		for (i = 0; i < St->Length; i++)
			rw[i] = (char)St[i];
		rw[i] = 0;
	}
}

void TimeStamp()
// printout the current MM/dd/yyyy hh:mm:ss AM/PM as a timestamp
{
	DateTime dt = DateTime::Now;
	String^ St = dt.ToString("MM/dd/yyyy hh:mm:ss tt");
 	char st[25];
	for (i = 0; i < St->Length; i++)
		st[i] = (char)St[i];
	st[i] = 0;
	cout << st << nL;
} // TimeStamp

/* end of file RomanNumC.cpp ----------------------------------------- */
